/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

/*!
  \file core.h
  \brief Core
  
  \author Frank Lemke
  \date 2006-03-02
*/

#ifndef CORE_H_
#define CORE_H_

#include <iterator>
#include <string>
#include <cstdlib>
#include "../core_I.h"
#include "fsm.h"

/*!
  \namespace std
  \brief Standard Library Namespace
*/


using namespace std;

/*!
  \brief Core

  The Core-object is the main object which consists of a vector of used FSMs
  and all Functions to add,delete or get FSMs.
*/

class Core : public Core_I {
 
 private:
  int numberOfFsms;
  vector <Fsm>::iterator fsmpointer;
  string name;
  vector <Fsm> fv;
  Fsm * currentFSM;

 public:
  Core();
  ~Core(){};

  /*!
    \brief projectName
    
    To change the name of the project.
    \param name New name of project.
  */

  void projectName(string pname);

  /*!
    \brief getprojectName
    
    To get the name of the project.
    \return string Name of project.
  */

  string getProjectName();

  /*!
    \brief addFsm
    
    Funktion to add a new empty FSM at the end of FSM-vector with
    automatical generated ID.
  */
  
  void addFsm();
  
  /*!
    \brief deleteFsm
    
    Funktion for delete FSMs by id.
    \param id ID to choose FSM for deletion.
  */
  
  void deleteFsm(int id);
  
  /*!
    \brief getFirstFsm
    
    This function checks if there is a FSM in the vector. If there exists one 
    the function makes the internal fsmpointer point to the first FSM in the
    vector and returns true. 
    \return bool True if there already exists a FSM.
  */
  
  bool getFirstFsm();
  
  /*!
    \brief getNextFsm
    
    Checks if there is a FSM stored after the current used one, in case of success
    sets the fsmpointer to the next FSM and returns true.
    /return bool True if there exists a FSM in the Fsm-vector after the 
    current used one.
  */
  
  bool getNextFsm();

  /*!
    \brief setCurrentFsm

    To set the current fsmpointer value to the value of variable id.
    \return return bool True if setting the FSM to the id value FSM
    and False if the id is greater than the number of available FSMs.
  */

  bool setCurrentFsm(int id);
  
  /*!
    \brief getCurrentFsm
    
    To give back current fsmpointer value which is set before by getFirstFsm or 
    getNextFsm.
    \return Fsm_I& Fsm_Interface reference.
  */
  
  Fsm_I& getCurrentFsm();
  


  /*!
    \brief getFsm  

    Function for pick FSM by ID and return a reference on it.
    \return Fsm_I& Fsm_Interface reference.
  */
  
  Fsm_I& getFsm(int i);

  /**
   * Says if there is at least one FSM available
   * If fsmpointer is not set, set it to first to avoid isFMS() call and then a getCurrentFSM that returns nothing valid
   *
   * @return true if there is a FSM, false if not
   */
  bool isFsm();

  /*!
    \brief reset

    Function to reset Core.
  */

  void reset();
};


#endif
