/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

/*!
  \file fsm.h
  \brief Fsm
  
  \author Frank Lemke
  \date 2006-02-28
*/

#ifndef FSM_H_
#define FSM_H_


#include <string>
#include <cstdlib>
#include <vector>
#include <iterator>
#include "condition.h"
#include "state.h"
#include "trans.h"
#include "hypertrans.h"
#include "link.h"
#include "../update_I.h"
#include "../fsm_I.h"

using namespace std;

/*!
  \brief Fsm-Object

  Object to store a complete FSM with all needed abilities..
*/

class Fsm : public Fsm_I {

 private:  
  int numberOfStates;
  int numberOfTrans;
  int numberOfHypertrans;
  int numberOfLinks;
  int numberOfParameters;
  int fid;
  int numberOfOutputs;
  int numberOfInputs;
  int resetstate;
  string fname;
  vector <State>::iterator statepointer;
  vector <Trans>::iterator transpointer;
  vector <Hypertrans>::iterator hypertranspointer;
  vector <Link>::iterator linkpointer;
  vector <Condition>::iterator parameterpointer;

 public:
  vector <State> sv;
  vector <Trans> tv;
  vector <Hypertrans> htv;
  vector <Link> lv;

  pair <string, string> globals;

  vector <Condition> parameter;
  vector <string> inputnames;
  vector <string> outputnames;
  vector <Update_I*> views;


  string itos(int i);
  Fsm(int f);
  ~Fsm(){};
  Fsm &operator=(const Fsm &fsm);
  int operator==(const Fsm &fsm) const;
  int operator<(const Fsm &fsm) const;

   /*!
    \brief clockName

    To change clock-name..
   */
  
  void clockName(string s);

  /*!
    \brief resetName

    To change reset-name..
   */
  
  void resetName(string s);
  

  /*!
    \brief registerView

    To register a new user.
  */
  
  void registerView(Update_I* u);

  /*!
    \brief signoffView

    To sign off user.
  */
  
  void signoffView(Update_I* u);

  /*!
    \brief update

    Update all registered users.
  */
  
  void update();

  /*!
    \brief addInputName
    
    Adds a new inputname at the end of inputvector.
  */
  
  void addInputName();

  /*!
    \brief addOutputName

    Adds a new outputname at the end of outputvector
  */
  
  void addOutputName();

  /*!
    \brief deleteInputName
    
    Delete last inputname of inputvector.
  */
  
  void deleteInputName();

  /*!
    \brief deleteOutputName

    Delete last outputname of outputvector.
  */
  
  void deleteOutputName();

  /*!
    \brief inputName

    Changes the inputname of a input.
    \param i Position of chosen input.
    \param name New name of input.
  */
  
  void inputName(int i, string name);

  /*!
    \brief outputName

    Changes the outputname of a output.
    \param i Position of chosen output.
    \param name New name of output.
  */
  
  void outputName(int i, string name);

 /*!
    \brief deleteInput

    Deletes input i.
    \param i Position of input for deleting.
  */

  void deleteInput(int i);

 /*!
    \brief deleteOutput

    Deletes output i.
    \param i Position of output for deleting.
  */
  
  void deleteOutput(int i);
  
  /*!
    \brief moveupInput
    
    Moves output i one step forward in inputvector.
    \param i Position of input for repositioning.
  */
  
  void moveupInput(int i);
  
  /*!
    \brief moveupOutput
    
    Moves output i one step forward in outputvector.
    \param i Position of output for repositioning.
  */
  
  void moveupOutput(int i);
  
  /*!
    \brief movedowninput
    
    Moves output i one step back in inputvector.
    \param i Position of input for repositioning.
  */
  
  void movedownInput(int i);
  
  /*!
    \brief movedownOutput
    
    Moves output i one step back in outputvector.
    \param i Position of output for repositioning.
  */
  
  void movedownOutput(int i);

  /*!
    \brief numberOfOuts

    Changes the number of outputbits to a new value.
    \param i New number of outputbits.
    \param defaultValue defines the default value of outputbits.
  */
  
  void numberOfOuts(int i, bool defaultValue);

  /*!
    \brief numberOfIns

    Changes the number of inputbits to new value.
    \param i New number of inputbits.
     \param defaultValue defines the default value of inputbits.
  */
  
  void numberOfIns(int i, int defaultValue);

  /*!
    \brief resetState.

    Sets new resetstate.
    \param id ID of new resetstate.
  */
  
  void resetState(int id);

  /*!
    \brief fsmName
    
    Changes name os Fsm.
    \param str New name for Fsm.
  */
  
  void fsmName(string str);

  /*!
    \brief addState

    Adds new State at the end of statevector of Fsm.
  */
  
  void addState ();

  /*!
    \brief deleteState.

    Delete State of Fsm by state-ID
    \param id ID for delete %State.
  */
  
  void deleteState(int id);

  /*!
    \brief modifyState

    Changes name of State by ID.
    \param id ID of a %State.
    \param name New name of %State.
  */
  
  void modifyState(int id, string name);

  /*!
    \brief modifyState 

    Changes the colour of a State.
    \param id ID of a %State.
    \param colour New colour for %State.
  */
  
  void modifyState(int id, unsigned int colour);

  /*!
    \brief modifyState 

    Function to change Position of State.
    \param id ID of a %State.
    \param posx New x-position of %State. 
    \param posy New y-position of %State.
  */
  
  void modifyState(int id, double posx, double posy);

  /*!
    \brief modifyState 

    Function to change outputbits of State.
    \param id ID of a %State.
    \param position Outputposition to change. 
    \param c New value for chosen outputbit.
  */
  
  void modifyState(int id, int position, char c);

  /*!
    \brief stateOutput 

    Function to change all outputbits of State.
    \param id ID of a %State.
    \param output New outputbits.
  */
  
  void stateOutput(int id, string output);

  /*!
    \brief stateLevel

    To change hierarchylevel of State.
    \param id ID of a %State.
    \param hierarchylevel New hierarchylevel for %State.
  */
  
  void stateLevel(int id, int hierarchylevel);

  /*!
    \brief addTrans

    Adds new Transition to Fsm.
    \param start Startpoint of transition.
    \param end Endpoint of transition.
  */
  
  void addTrans(int start, int end);

  /*!
    \brief deleteTrans

    Delete Trans by ID.
    \param id ID of a Transition.
  */
  
  void deleteTrans(int id);

  /*!
    \brief modifyTrans

    Change colour of Trans.
    \param id ID of %Trans.
    \param colour New Transition colour.
  */
  
  void modifyTrans(int id, int colour);

  /*!
    \brief modifyTransLevel

    Change level of Trans.
    \param id ID of %Trans.
    \param newlevel New Transition level.
  */

  void modifyTransLevel(int id, int newlevel);

  /*!
    \brief modifyTrans

    Activate or deactivate defaultfunction for Transition.
    \param id ID of %Trans.
    \param b True activates default.
  */
  
  void modifyTrans(int id, bool b);
  
  /*!
    \brief modifyTrans
    
    To change name of Transition.
    \param id ID of %Trans.
    \param name New name for Transition.
  */
  
  void modifyTrans(int id, string name);
  
  /*!
    \brief modifyTrans

    Changes position of Transname.
    \param id ID of %Trans.
    \param posx x-position of Transname.
    \param posy y-position of Transname.
    \param level Level for Transname placement.
  */
  
  void modifyTrans(int id, double posx, double posy, int level);

  /*!
    \brief modifyTrans

    Changes startstate and endstate of Trans.
    \param id ID of %Trans.
    \param start Startstate-ID of %Trans.
    \param end Endstate-ID of %Trans.
  */
  
  void modifyTrans(int id, int start, int end);

  /*!
    \brief modifyTrans

    To change inputvalue of chosen Condition.
    \param id ID of %Trans.
    \param cid ID of %Condition.
    \param position Position of the inputbit to change.
    \param value New value for inputbit.
  */
  
  void modifyTrans(int id, int cid, int position, char value);

  /*!
    \brief modifyTransConditionValue
    
    To change complete inputvalue of chosen Condition.
    \param id ID of %Trans.
    \param cid ID of %Condition.
    \param value New value for inputbits.
  */

  void modifyTransConditionValue(int id, int cid, string value) ;

  /*!
    \brief modifyTrans

    To change name of chosen Condition.
    \param id ID of %Trans.
    \param cid ID of %Condition.
    \param name New name for %Condition.
  */
  
  void modifyTrans(int id, int cid, string name);

  /*!
    \brief addConditionToTrans

    Adds new Condition to Trans.
    \param id ID of Transition.
  */
  
  void addConditionToTrans(int id);

  /*!
    \brief deleteConditionOfTrans

    Delete Condition of a Transition by ids.
    \param id ID of Transition.
    \param cid ID of %Condition.
  */
  
  void deleteConditionOfTrans(int id, int cid);

  /*!
    \brief addTrackpointToTrans

    Adds new Trackpoint to Trans.
    \param id ID of Transition.
    \param posid ID of Position.
    \param b True to place in front of posid.
    \param posx X-position.
    \param posy Y-position.
    \param hl Hierarchylevel.
  */
  
  void addTrackpointToTrans(int id, int posid, bool b, double posx, double posy, int hl);

  /*!
    \brief deleteTrackpointOfTrans

    Delete Trackpoint of a Transition by ids.
    \param id ID of Transition.
    \param trid ID of %Trackpoint.
  */
  
  void deleteTrackpointOfTrans(int id, int trid);

  /*!
    \brief modifyTrans

    Chnage Position of a Trackpoint.
    \param id ID of Transition.
    \param trid ID of %Trackpoint. 
    \param posx X-position of %Trackpoint.
    \param posy Y-position of %Trackpoint.
  */
  
  void modifyTrans(int id, int trid, double posx, double posy);

  /*!
    \brief modifyTrans

    Activates or deactivetes the function to use a Link and sets
    the target Link of a Trackpoint.
    \param id ID of Transition.
    \param trid ID of %Trackpoint.
    \param b Activates %Link.
    \param linkid ID of target-%Link
  */
  
  void modifyTrans(int id,int trid, bool b, int linkid);

  /*!
    \brief modifyTrans

    Activates or deactivetes the function to use a Trackpoint as a Join.

    \param id ID of Transition.
    \param trid ID of %Trackpoint.
    \param b Activates or deactivates Join.
  */
  
  void modifyTrans(int id,int trid, bool b);

  /*!
    \brief transTrackpointColor

    Changes the color of a Trackpoint.
    \param id ID of Transition.
    \param trid ID of %Trackpoint.
    \param color New color for %Trackpoint.
  */
  
  void modifyTransTrackpointColor(int id, int trid, int color);

  /*!
    \brief transLevel

    Makes a Trackpoint become a hierarchical-state.
    \param id ID of Transition.
    \param trid ID of %Trackpoint.
    \param hldir Hierarchylevel of next object.
  */
  
  void transLevel(int id, int trid, int hldir);

  /*!
    \brief addHypertrans

    Adds a new Hypertrans to Fsm.
    \param type Shows type of %Hypertrans.
    \param posx X-position.
    \param posy Y-position
    \param target target of %Hypertrans.
    \param level Hierarchylevel of %Hypertrans.
  */
  
  void addHypertrans(bool type, double posx, double posy, int target, int level);

  /*!
    \brief deleteHypertrans

    Delete Hypertrans by ID.
    \param id ID of Hypertransition.
  */
  
  void deleteHypertrans(int id);

  /*!
    \brief modifyHypertrans

    To change the colour of Hypertrans.
    \param id ID of Hypertransition.
    \param colour New colour.
  */
  
  void modifyHypertrans(int id, int colour);

  /*!
    \brief modifyHypertrans

    Change position of Hypertrans.
    \param id ID of Hypertransition.
    \param posx New x-position.
    \param posy New y-position.
  */
  
  void modifyHypertrans(int id, double posx, double posy);

  /*!
    \brief modifyHypertrans
    
    Change Position and Target of Hypertrans.
    \param id ID of Hypertransition.
    \param posx New x-position.
    \param posy New y-position.
    \param target New target-State.
  */
  
  void modifyHypertrans(int id, double posx, double posy, int target);

  /*!
    \brief modifyHypertrans

    Sets type of Hypertrans.
    \param id ID of Hypertransition.
    \param b True if all_States, false ifrest_states
  */
  
  void modifyHypertrans(int id, bool b); 

  /*!
    \brief modifyHypertrans
    
    Changes Condition inputbit at chosen position.
    \param id ID of Hypertransition.
    \param cid ID of %Condition
    \param position Position for inputbit.
    \param value New Value for chosen inputbit.
  */
  
  void modifyHypertrans(int id, int cid, int position, char value);

  /*!
    \brief modifyHypertransConditionValue
    
    Changes all Condition inputbits.
    \param id ID of Hypertransition.
    \param cid ID of %Condition
    \param value New Value for inputbits.
  */
  
  void modifyHypertransConditionValue(int id, int cid, string value);

  /*!
    \brief modifyHypertrans

    Change name of Condition of Hypertrans.
    \param id ID of Hypertransition.
    \param cid ID of %Condition
    \param name New name of %Condition.
  */
  
  void modifyHypertrans(int id, int cid, string name);

  /*!
    \brief addConditionToHypertrans

    Adds new Condition to Hypertrans.
    \param id ID of Hypertransition.
  */
  
  void addConditionToHypertrans(int id);

  /*!
    \brief deleteConditionOfHypertrans

    Delete Condition of Hypertrans by ID.
    \param id ID of Hypertransition.
    \param cid ID of %Condition
  */
  
  void deleteConditionOfHypertrans(int id, int cid);

  /*!
    \brief hypertransLevel
    
    Changes hierarchylevel of Hypertrans.
    \param id ID of Hypertransition.
    \param hierarchylevel New hierarchylevel.
  */
  
  void hypertransLevel(int id, int hierarchylevel);

  /*!
    \brief addLink

    Adds Link to Fsm.
    \param posx X-position.
    \param posy Y-position.
    \param target Target of %Link.
    \param level Hierarchylevel of %Link.
  */
  
  void addLink(double posx, double posy, int target, int level);

  /*!
    \brief deleteLink

    Delete Link by ID.
    \param id ID of %Link.
  */
  
  void deleteLink(int id);

  /*!
    \brief modifyLink

    Changes colour of Link.
    \param id ID of %Link.
    \param colour
  */
  
  void modifyLink(int id, int colour);

  /*!
    \brief modifyLink

    Changes the position of the Link.
    \param id ID of %Link.
    \param posx New x-position.
    \param posy New y-position.
  */
  
  void modifyLink(int id, double posx, double posy);

  /*!
    \brief linkLevel

    Changes hierarchylevel of Link.
    \param id ID of %Link.
    \param hierarchylevel New hierarchylevel.
  */
  
  void linkLevel(int id, int hierarchylevel);

  /*!
    \brief linkGoal

    Changes target of Link.
    \param id ID of %Link.
    \param new target.
  */
  
  void linkGoal(int id, int target);

  /*!
    \brief addParameter

    Adds Parameter to the end of parametervector.
  */
  
  void addParameter();

  /*!
    \brief modifyParameter

    
    \param id ID of %Parameter.
    \param position Position of inputbit.
    \param value New value for chosen inputbit.
  */
  
  void modifyParameter(int id, int position, char value);

  /*!
    \brief modifyParameter

    Change name of Parameter.
    \param id ID of %Parameter.
    \param name New name for %Parameter.
  */
  
  void modifyParameter(int id, string name);

  /*!
    \brief deleteParameter

    Delete Parameter by ID.
    \param id ID of %Parameter.
  */
  
  void deleteParameter(int id);

  /*!
    \brief getOutputName

    To get a outputname.
    \param pos position of output.
    \return string Name of output.
  */
  
  string getOutputName(int pos);

  /*!
    \brief getInputName

    To get a inputname.
    \param pos position of input.
    \return string Name of input.
  */
  
  string getInputName(int pos);

  /*!
    \brief getFsmID

    To get ID of Fsm.
    \return int ID of %Fsm.
  */
  
  int getFsmID();

  /*!
    \brief getNumberOfInputs

    Get number of inputbits.
    \return int Number of inputbits.
  */
  
  int getNumberOfInputs();

  /*!
    \brief getNumberOfOutputs

    Get number of outputbits.
    \return int Number of outputbits.
  */
  
  int getNumberOfOutputs();

  /*!
    \brief getResetState

    To get resetstate
    \return int ID of resetstate
  */
  
  int getResetState();

  /*!
    \brief getFsmName

    Get name of Fsm.
    \return string Name of Fsm.
  */
  
  string getFsmName();

  /*!
    \brief getFirstParameter
    
    Checks if Parametervector is empty, if yes parameterpointer is set to
    first Parameter.
    \return bool True if %Parameter exists.
  */
  
  
  bool getFirstParameter();

  /*!
    \brief getNextParameter

    If there is a following Parameter in the parametervector, it sets the
    parameterpointer to the next Parameter.
    \return bool True if next Parameter exists.
  */
  
  bool getNextParameter();

  /*!
    \brief getParameterName

    Get name of current Parameter.
    \return string Name of current Parameter.
  */
  
  string getParameterName();

  /*!
    \brief getParameterValue

    Get value of current Parameter.
    \return string Value of current Parameter.
  */
  
  string getParameterValue();

  /*!
    \brief getParameterID

    Get ID of current Parameter.
    \return int ID of current Parameter.
  */
  
  int getParameterID();

  /*!
    \brief getFirstState
    
    Checks if statevector is empty, if yes statepointer is set to
    first State.
    \return bool True if %State exists.
  */
   
  bool getFirstState();

  /*!
    \brief getNextState

    If there is a following State in the statevector, it sets the
    statepointer to the next State.
    \return bool True if next %State exists.
  */
  
  bool getNextState();

 /*!
    \brief getStatebyID

    If there is a State in the statevector with chosen ID it becomes current
    selected State.
    \param id ID of wanted %State
    \return bool True if the %State with chosen id exists.
  */

  bool getStatebyID(int id);
  
  /*!
    \brief getStateName

    Get name of current State.
    \return string Name of current %State.
  */

  string getStateName();

  /*!
    \brief getStatePosx

    Get x-position of current State.
    \return double X-position of current %State.
  */
  
  double getStatePosx();

  /*!
    \brief getStatePosy

    Get y-position of current State.
    \return double Y-position of current %State.
  */
  
  double getStatePosy();

  /*!
    \brief getStateID

    Get ID of current State.
    \return int ID of current %State.
  */
  
  int getStateID();

  /*!
    \brief getStateColor

    Get color of current State.
    \return int Color of current %State.
  */
  
  unsigned int getStateColor();

  /*!
    \brief getStatelevel

    Get hierarchylevel of current State.
    \return int Hierarchylevel of current %State.
  */
  
  int getStateLevel();

  /*!
    \brief getStateOutput

    Get output of current State.
    \return string Output of current %State.
  */
  
  string getStateOutput();

  /*!
    \brief getStateOutputBit

    Get output bit of current State at position pos.
    \return char Output bit of current %State at position pos.
  */

  char getStateOutputBit(int pos);

  /*!
    \brief getFirstTrans
    
    Checks if transvector is empty, if yes transpointer is set to
    first Trans.
    \return bool True if %Trans exists.
  */ 
  
  bool getFirstTrans();

  /*!
    \brief getNextTrans

    If there is a following Trans in the transvector, it sets the
    transpointer to the next Trans.
    \return bool True if next %Trans exists.
  */
  
  bool getNextTrans();
  
  /*!
    \brief getTransbyID
    
    If there is a Transition in the transition-vector with chosen ID 
    it becomes current selected Transition.
    \param id ID of wanted Transition.
    \return bool True if the Transition with chosen id exists.
  */
  
  bool getTransbyID(int id);
  
  /*!
    \brief getTransStart

    Get Startstate of current Trans.
    \return int Startstate of current %Trans.
  */
  
  int getTransStart();

  /*!
    \brief getTransEnd

    Get Endstate of current Trans.
    \return int Endstate of current %Trans.
  */
  
  int getTransEnd();

  /*!
    \brief getTransName

    Get Name of current Trans.
    \return string Name of current %Trans.
  */
  
  string getTransName();

  /*!
    \brief getTransID

    Get ID of current Trans.
    \return int ID of current %Trans.
  */
  
  int getTransID();

  /*!
    \brief getTransColor

    Get Color of current Trans.
    \return int Color of current %Trans.
  */
  
  int getTransColor();

  /*!
    \brief getTransDefault

    Shows if current Trans is a default Transition.
    \return bool True if default is set in current %Trans.
  */
  
  bool getTransDefault();

  /*!
    \brief getTransTextPosx

    Get x-position of current Transname.
    \return double X-position of current Transname.
  */
  
  double getTransTextPosx();

  /*!
    \brief getTransTextPosy

    Get y-position of current Transname.
    \return double Y-position of current Transname.
  */
  
  double getTransTextPosy();

  /*!
    \brief getTransLevel

    Get Level of current Transname.
    \return int Level of current Transname.
  */
  
  int getTransLevel();

  /*!
    \brief getTransFirstCondition
    
    Starts the getFirstCondition function of current Trans. 
    \return bool Result of getFirstCondition of current Trans.
  */ 
  

  bool getTransFirstCondition();

  /*!
    \brief getTransNextCondition
    
    Starts the getNextCondition function of current Trans. 
    \return bool Result of getNextCondition of current Trans.
  */
  
  bool getTransNextCondition();
 
  /*!
    \brief getTransConditionbyID
    
    If there is a Condition in the current Transition with chosen ID 
    it becomes current selected Condition for this Transition.
    \param id ID of wanted Condition.
    \return bool True if Condition exists in current Transition.
  */
  
  bool getTransConditionbyID(int id);

  /*!
    \brief getTransConditionName
    
    Starts the getConditionName function of current Trans. 
    \return string Result of getConditionName of current Trans.
  */
  
  string getTransConditionName();

  /*!
    \brief getTransConditionValue
    
    Starts the getConditionValue function of current Trans. 
    \return string Result of getConditionValue of current Trans.
  */
  
  string getTransConditionValue();

  /*!
    \brief getTransConditionBit

    Get input bit of the current Condition of the current Transition at position pos.
    \return char Input bit of current Condition of the current Transition at position pos.
  */

  char getTransConditionBit(int pos);

  /*!
    \brief getTransConditionID
    
    Starts the getConditionID function of current Trans. 
    \return int Result of getConditionID of current Trans.
  */
  
  int getTransConditionID();

  /*!
    \brief getTransFirstTrackpoint
    
    Starts the getFirstTrackpoint function of current Trans. 
    \return bool Result of getFirstTrackpoint of current Trans.
  */
  
  bool getTransFirstTrackpoint();

  /*!
    \brief getTransNextTrackpoint
    
    Starts the getNextTrackpoint function of current Trans. 
    \return bool Result of getNextTrackpoint of current Trans.
  */
  
  bool getTransNextTrackpoint();

  /*!
    \brief getTransTrackpointbyID
    
    If there is a Trackpoint in the current Transition with chosen ID 
    it becomes current selected Trackpoint for this Transition.
    \param id ID of wanted Trackpoint.
    \return bool True if Trackpoint exists in current Transition.
  */
  
  bool getTransTrackpointbyID(int id);

  /*!
    \brief getTransTrackpointPosx
    
    Starts the getTrackpointPosx function of current Trans. 
    \return double Result of getTrackpointposx of current Trans.
  */
  
  double getTransTrackpointPosx();

  /*!
    \brief getTransTrackpointPosy
    
    Starts the getTrackpointPosy function of current Trans. 
    \return double Result of getTrackpointposy of current Trans.
  */
  
  double getTransTrackpointPosy();

  /*!
    \brief getTransTrackpointHldirection
    
    Starts the getTrackpointHldirection function of current Trans. 
    \return int Result of getTrackpointHldirection of current Trans.
  */
  
  int getTransTrackpointHldirection();

  /*!
    \brief getTransTrackpointColor
    
    Starts the getTrackpointColor function of current Trans. 
    \return int Result of getTrackpointLevel of current Trans.
  */
  
  int getTransTrackpointColor();

  /*!
    \brief getTransTrackpointLevel
    
    Starts the getTrackpointLevel function of current Trans. 
    \return int Result of getTrackpointLevel of current Trans.
  */
  
  int getTransTrackpointLevel();

  /*!
    \brief getTransTrackpointID
    
    Starts the getTrackpointID function of current Trans. 
    \return int Result of getTrackpointID of current Trans.
  */
  
  int getTransTrackpointID();

  /*!
    \brief getTransTrackpointLink
    
    Starts the getTrackpointLink function of current Trans. 
    \return bool Result of getTrackpointLink of current Trans.
  */
  
  bool getTransTrackpointLink();

  /*!
    \brief getTransTrackpointLinkID
    
    Starts the getTrackpointLinkID function of current Trans. 
    \return int Result of getTrackpointLinkID of current Trans.
  */
  
  int getTransTrackpointLinkID();

  /*!
    \brief getTransTrackpointJoin
    
    Starts the getTrackpointJoin function of current Trans. 
    \return bool Result of getTrackpointJoin of current Trans.
  */
  
  bool getTransTrackpointJoin();

  /*!
    \brief getFirstLink
    
    Checks if linkvector is empty, if yes linkpointer is set to
    first Link.
    \return bool True if %Link exists.
  */
  
  
  bool getFirstLink();

  /*!
    \brief getNextLink

    If there is a following Link in the linkvector, it sets the
    linkpointer to the next Link.
    \return bool True if next %Link exists.
  */
  
  bool getNextLink();
  
  /*!
    \brief getLinkbyID
    
    If there is a Link in the link-vector with chosen ID 
    it becomes current selected Link.
    \param id ID of wanted Link.
    \return bool True if Link with chosen id exists.
  */
  
  bool getLinkbyID(int id);

  /*!
    \brief getLinkID

    Get ID of current Link.
    \return int ID of current %Link.
  */
  
  int getLinkID();

  /*!
    \brief getLinkGoal

    Get goal of current Link.
    \return int Goal of current %Link.
  */
  
  int getLinkGoal();

  /*!
    \brief getLinkColor

    Get Color of current Link.
    \return int Color of current %Link.
  */
  
  int getLinkColor();

  /*!
    \brief getLinkLevel

    Get heirarchylevel of current Link.
    \return int Hierarchylevel of current %Link.
  */
  
  int getLinkLevel();

  /*!
    \brief getLinkPosx

    Get x-position of current Link.
    \return double X-position of current %Link.
  */
  
  double getLinkPosx();

  /*!
    \brief getLinkPosy

    Get y-position of current Link.
    \return double Y-position of current %Link.
  */
  
  double getLinkPosy();

  /*!
    \brief getFirstHypertrans
    
    Checks if hypertransvector is empty, if yes hyperpointer is set to
    first Hypertrans.
    \return bool True if %Hypertrans exists.
  */
   
  bool getFirstHypertrans();

  /*!
    \brief getNextHypertrans

    If there is a following Hypertrans in the hypertransvector, it sets the
    hypertranspointer to the next Hypertrans.
    \return bool True if next %Hypertrans exists.
  */
  
  bool getNextHypertrans();
  
  /*!
    \brief getHypertransbyID
    
    If there is a Hypertransition in the hypertransition-vector with chosen ID 
    it becomes current selected Hypertransition.
    \param id ID of wanted Hypertransition.
    \return bool True if Hypertransition with chosen id exists.
  */
  
  bool getHypertransbyID(int id);

  /*!
    \brief getHypertransColor

    Get Color of current Hypertrans.
    \return int Color of current %Hypertrans.
  */
  
  int getHypertransColor();

  /*!
    \brief getHypertransID

    Get ID of current Hypertrans.
    \return int ID of current %Hypertrans.
  */
  
  int getHypertransID();

  /*!
    \brief getHypertransLevel

    Get hierarchylevel of current Hypertrans.
    \return int Hierarchylevel of current %Hypertrans.
  */
  
  int getHypertransLevel();

  /*!
    \brief getHypertransGoal

    Get goal of current Hypertrans.
    \return int Goal of current %Hypertrans.
  */
  
  int getHypertransGoal();

  /*!
    \brief getHypertransGoal

    Get type of current Hypertrans.
    \return bool Type of current %Hypertrans.
  */
  
  bool getHypertransType();

  /*!
    \brief getHypertransPosx

    Get x-position of current Hypertrans.
    \return double X-position of current %Hypertrans.
  */
  
  double getHypertransPosx();

  /*!
    \brief getHypertransPosy

    Get y-position of current Hypertrans.
    \return double Y-position of current %Hypertrans.
  */
  
  double getHypertransPosy();

  /*!
    \brief getHypertransFirstCondition
    
    Starts the getHypertransFirstCondition function of current Trans. 
    \return bool Result of getHypertransFirstCondition of current Trans.
  */
  
  bool getHypertransFirstCondition();

  /*!
    \brief getHypertransNextCondition
    
    Starts the getHypertransNextCondition function of current Trans. 
    \return bool Result of getHypertransNextCondition of current Trans.
  */
  
  bool getHypertransNextCondition();

  /*!
    \brief getHypertransConditionbyID
    
    If there is a Condition in the current Hypertransition with chosen ID 
    it becomes current selected Condition for this Hypertransition.
    \param id ID of wanted Condition.
    \return bool True if Condition exists in current Hypertransition.
  */
  
  bool getHypertransConditionbyID(int id);

  /*!
    \brief getHypertransConditionName
    
    Starts the getHypertransConditionName function of current Trans. 
    \return string Result of getHypertransConditionName of current Trans.
  */
  
  string getHypertransConditionName();

  /*!
    \brief getHypertransConditionValue
    
    Starts the getHypertransConditionValue function of current Trans. 
    \return string Result of getHypertransConditionValue of current Trans.
  */
  
  string getHypertransConditionValue();

  /*!
    \brief getHypertransConditionID
    
    Starts the getHypertransConditionID function of current Trans. 
    \return int Result of getHypertransConditionID of current Trans.
  */
  
  int getHypertransConditionID();


 /*!
    \brief optimize
    
    Funktion for optimization of State and Link ID's.
  */
  
  void optimize();
  
   /*!
    \brief clockName

    To change clock-name..
   */
  
  string getClockName();

  /*!
    \brief resetName

    To change reset-name..
   */
  
  string getResetName();
};

#endif

