/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

/*!
  \file trans.h
  \brief Trans
  
  \author Frank Lemke
  \date 2006-02-28
*/

#ifndef TRANS_H_
#define TRANS_H_

#include <vector>
#include <list>
#include <iterator>
#include <string>
#include <cstdlib>
#include <utility>

#include "trackpoint.h"
#include "condition.h"

using namespace std;

/*!
  \brief Trans-Object

  Object to store Transitions with all needed parameters.
*/

class Trans {

 private:
  vector <Condition>::iterator conditionpointer;
  list <Trackpoint>::iterator trackpointpointer;
  int numberOfConditions;
  int numberOfTrackpoints;
 public:
  string tname;
  int tid;
  int start;
  int end;
  bool d; // default condition
  vector <Condition> cv;
  list <Trackpoint> tl;
  int color;
  int level;
  pair <double, double> textpos; 

  Trans(int id, int s, int e);
  ~Trans(){};
  string itos(int i);
  Trans &operator=(const Trans &tra);
  int operator==(const Trans &tra) const;
  int operator<(const Trans &tra) const;

  /*!
    \brief addCondition

    Function to add Condition to Trans.
    \param number Number of inputbits.
  */
 
  void addCondition(int number);

  /*!
    \brief deleteCondition

    Delete Condition by ID.
    \param id ID to delete associated %Condition.
  */
 
  void deleteCondition(int id);

  /*!
    \brief modifyCondition

    Change value of a inputbit.
    \param id ID of a %Condition. 
    \param position Position of the bit you want to change.
    \param value New value for choosen bit.
  */
 
  void modifyCondition(int id, int position, char value);

  /*!
    \brief modifyCondition

    Function to change inputparameters of Condition.
    \param id ID of a %Condition.
    \param name New name for %Condition.
  */
 
  void modifyCondition(int id, string name);

  /*!
    \brief addTrackpoint

    Function to add Trackpoints to Trans. Trackpoint can be added at the 
    beginning by passing a negativ value to posid parameter or at the end by
    passing a value greater then 999 to posid. To place new Trackpoint before
    a special chosen position set parameter b to true, false will place it
    after it.
    \param posid Position for %Trackpoint insertion.
    \param b Inserts the %Trackpoint after the chosen position for false.
    \param posx X-position of new %Trackpoint.
    \param posy Y-position of new %Trackpoint.
    \param hl hierarchylevel of new %Trackpoint.
  */
 
  void addTrackpoint(int posid, bool b, double posx, double posy, int hl);

  /*!
    \brief deleteTrackpoint

    Delete Trackpoint by ID.
    \param id ID to delete associated %Trackpoint.
  */
 
  void deleteTrackpoint(int id);

  /*!
    \brief modifyTrackpoint

    This function will change the position of a Trackpoint.
    \param id ID to choose %Trackpoint.
    \param posx New x-position.
    \param posy New y-position.
  */
 
  void modifyTrackpoint(int id, double posx, double posy);

  /*!
    \brief modifyTrackpoint

    Function to connect or deconnect a Trackpoint to a Link.
    \param id ID to choose %Trackpoint.
    \param b aktivates or deactivates the connection to a %Link.
    \param linkid ID of connected %Link.
  */
 
  void modifyTrackpoint(int id, bool b, int linkid);

  /*!
    \brief modifyTrackpoint

    Function to set the join flag for a Trackpoint. This trackpoint becomes a joining point.
    \param id ID to choose %Trackpoint.
    \param b aktivates or deactivates Join.
  */

  void modifyTrackpoint(int id, bool b);
  
  /*!
    \brief modifyTrackpoint

    Changes the color of the Trackpoint.
    \param id ID to choose %Trackpoint. 
    \param color New color for %Trackpoint.
  */

  void modifyTrackpoint(int id, int color);

  /*!
    \brief trackpointLevel

    Changes Trackpoint to Hierachical-State if hldir differs from hierarchy
    level of Trackpoint. Value shows the level hierarchical-state goes to.
    \param id ID to choose %Trackpoint. 
    \param hldir Shows hierachical level change.
  */

  void trackpointLevel(int id, int hldir);


  /*!
    \brief getFirstCondition

    This function checks if there is a Condition. In case of success it sets
    the conditonpointer to the first one.
    \return bool True if %Condition-vector is not empty.
  */
 
  bool getFirstCondition();

  /*!
    \brief getNextCondition

    Checks for following Conditions if there is one it makes the conditionpointer
    point to the next one in the vector.
    \return bool True if there is a %Condition after the current one.
  */
 
  bool getNextCondition();

  /*!
    \brief getConditionName
    
    To get the name of current Condition.
    \return string Conditionname 
  */
 
  string getConditionName();

  /*!
    \brief getConditionValue

    To get the value of current Condition.
    \return string Conditionvalue 
  */
 
  string getConditionValue();

  /*!
    \brief getConditionID

    To get the ID of current Condition.
    \return int ID
  */
 
  int getConditionID();

  /*!
    \brief getFirstTrackpoint

    This function checks if there is a Trackpoint. In case of success it sets
    the trackpointpointer to the first one.
    \return bool True if %Trackpoint-vector is not empty.
  */
 
  bool getFirstTrackpoint();

  /*!
    \brief getNextTrackpoint

    Checks for following Trackpoints if there is one it makes the trackpoint-
    pointer point to the next one in the vector.
    \return bool True if there is a %Trackpoint after the current one.
  */
 
  bool getNextTrackpoint();

  /*!
    \brief getTrackpointPosx

    To get the x-position of current Trackpoint.
    \return double X-position.
  */
 
  double getTrackpointPosx();

  /*!
    \brief getTrackpointPosy

    To get the y-position of current Trackpoint.
    \return double Y-position.
  */
 
  double getTrackpointPosy();

  /*!
    \brief getTrackpointHldirection

    To get the hierarchylevel where current Trackpoint goes to.
    \return int Hierarchylevel of next object.
  */
 
  int getTrackpointHldirection();

  /*!
    \brief getTrackpointColor

    To get the color of current Trackpoint.
    \return int color.
  */
 
  int getTrackpointColor();

  /*!
    \brief getTrackpointLevel

    To get the hierarchylevel of current Trackpoint.
    \return int Hierarchylevel.
  */
 
  int getTrackpointLevel();

  /*!
    \brief getTrackpointLevel

    To get the ID of current Trackpoint.
    \return int ID of %Trackpoint.
  */
 
  int getTrackpointID();

  /*!
    \brief getTrackpointLink

    To show if current Trackpoint is connected to a Link.
    \return bool True if %Trackpoint is connected to a %Link.
  */
 
  bool getTrackpointLink();

  /*!
    \brief getTrackpointLinkID

    To get the ID of Link current Trackpoint is connected to.
    \return int ID of connected %Link.
  */
 
  int getTrackpointLinkID();

  /*!
    \brief getTrackpointJoin

    To show if the current Trackpoint is a Joinpoint.
    \return bool True if this %Trackpont is a Join.
  */

  bool getTrackpointJoin();
};
 
#endif
